# Copyright 2014 MINES ParisTech
#
# This file is part of LinPy.
#
# LinPy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# LinPy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LinPy.  If not, see <http://www.gnu.org/licenses/>.

import functools
import unittest

from ..linexprs import symbols
from ..polyhedra import *
from .libhelper import requires_sympy


class TestPolyhedron(unittest.TestCase):

    def setUp(self):
        x, y = symbols('x y')
        self.square = Polyhedron(inequalities=[x, 1 - x, y, 1 - y])

    def test_symbols(self):
        self.assertTupleEqual(self.square.symbols, symbols('x y'))

    def test_dimension(self):
        self.assertEqual(self.square.dimension, 2)

    def test_repr(self):
        self.assertEqual(repr(self.square),
            "And(0 <= x, x <= 1, 0 <= y, y <= 1)")

    def test_fromstring(self):
        self.assertEqual(Polyhedron.fromstring('{x >= 0, -x + 1 >= 0, '
            'y >= 0, -y + 1 >= 0}'), self.square)

    def test_isempty(self):
        self.assertFalse(self.square.isempty())

    def test_isuniverse(self):
        self.assertFalse(self.square.isuniverse())

    @requires_sympy
    def test_fromsympy(self):
        import sympy
        sp_x, sp_y = sympy.symbols('x y')
        self.assertEqual(Polyhedron.fromsympy((sp_x >= 0) & (sp_x <= 1) &
            (sp_y >= 0) & (sp_y <= 1)), self.square)

    @requires_sympy
    def test_tosympy(self):
        import sympy
        sp_x, sp_y = sympy.symbols('x y')
        self.assertEqual(self.square.tosympy(),
            sympy.And(-sp_x + 1 >= 0, -sp_y + 1 >= 0, sp_x >= 0, sp_y >= 0))


class TestEmpty:

    def test_repr(self):
        self.assertEqual(repr(Empty), 'Empty')

    def test_isempty(self):
        self.assertTrue(Empty.isempty())

    def test_isuniverse(self):
        self.assertFalse(Empty.isuniverse())


class TestUniverse:

    def test_repr(self):
        self.assertEqual(repr(Universe), 'Universe')

    def test_isempty(self):
        self.assertTrue(Universe.isempty())

    def test_isuniverse(self):
        self.assertTrue(Universe.isuniverse())
